/*******************************************************************
 *
 *	File:		GnuBoy.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <e32std.h>
#include <f32file.h>
#include <e32svr.h>
#include <estlib.h>
#include <eikdll.h>
#include <w32std.h>


#include "Game.h"
#include "GnuBoy.h"
#include "GlobalData.h"
#include "platform.h"
#include "Video.h"

/*********************************************
 *
 * Game options
 *
 *********************************************/

extern "C"
{

int PsionDoubleScreen = 0;
int PsionBpp = 8;

rcvar_t gameoptions_exports[] =
{
	RCV_BOOL("fitscreen", &PsionDoubleScreen),
	RCV_END
};


}

/*********************************************
 *
 * Keyboard handler
 *
 *********************************************/

class CKeyboardHandler : public CBase, public MKeyboardObserver
{
public:
	~CKeyboardHandler(){};
	static CKeyboardHandler* NewL();

protected:
	CKeyboardHandler(){};
	void ConstructL();

	TInt MapScanCode(TInt aScancode);
	void PostEvent(TInt aScancode, TBool aUpEvent);

	virtual void KeyDownEvent(TInt aScancode);
	virtual void KeyUpEvent(TInt aScancode);

	static const TInt sKeyMapTable[][2];
};

const TInt CKeyboardHandler::sKeyMapTable[][2] = 
{
	{EStdKeyLeftArrow, K_LEFT},
	{EStdKeyRightArrow, K_RIGHT},
	{EStdKeyUpArrow, K_UP},
	{EStdKeyDownArrow, K_DOWN},
	{EStdKeyEscape, K_ESC},
	{EStdKeyEnter, K_ENTER},
	{EStdKeySpace, K_SPACE},

	{EStdKeyLeftShift, K_SHIFT},
	{EStdKeyRightShift, K_CTRL},
	{EStdKeyLeftFunc, K_ALT},

	{EStdKeyTab, K_TAB},
	{EStdKeyBackspace, K_BS},
	{EStdKeyDelete, K_DEL},

	{EStdKeyMinus, K_MINUS},
	{EStdKeyEquals, K_EQUALS},
	{EStdKeyForwardSlash, K_SLASH},
	{EStdKeyBackSlash, K_BSLASH},
	{EStdKeySemiColon, K_SEMI},

	{0, 0}
};


CKeyboardHandler* CKeyboardHandler::NewL()
{
	CKeyboardHandler*	self = new (ELeave) CKeyboardHandler;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();		// self
	return self;
}

void CKeyboardHandler::ConstructL()
{
}

TInt CKeyboardHandler::MapScanCode(TInt aScancode)
{
	int i;
	for (i = 0; sKeyMapTable[i][0]; i++)
		if (sKeyMapTable[i][0] == aScancode)
			return sKeyMapTable[i][1];
	return 0;
}

void CKeyboardHandler::KeyDownEvent(TInt aScancode)
{
	PostEvent(MapScanCode(aScancode), EFalse);
}

void CKeyboardHandler::KeyUpEvent(TInt aScancode)
{
	PostEvent(MapScanCode(aScancode), ETrue);
}


void CKeyboardHandler::PostEvent(TInt aScancode, TBool aUpEvent)
{
	if (aScancode)
	{
		event_t ev;
		ev.type = aUpEvent ? EV_RELEASE : EV_PRESS;
		ev.code = aScancode;
		ev_postevent(&ev);
	}
}


void sys_sleep(int us)
{
}

int sys_elapsed(unsigned *cl)
{
	return 0;
}

int pcm_submit()
{
	return 0;
}

void pcm_close()
{
}

void pcm_init()
{
}


void ev_poll()
{
}

extern "C"
{
rcvar_t pcm_exports[] =
{
	RCV_END
};

rcvar_t joy_exports[] =
{
	RCV_END
};

rcvar_t vid_exports[] =
{
	RCV_END
};
}

struct fb fb;
struct pcm pcm;

void *sys_timer()
{
	return NULL;
}

void sys_checkdir(char *path, int wr)
{
}

void sys_initpath(char *exe)
{
}

void sys_sanitize(char *s)
{
}


static CGameWindow*			sGameWindow = NULL;
static CKeyboardHandler*	sKeyboardHandler = NULL;

void vid_preinit()
{
}

void vid_init()
{
	TSize	gameSize(320, 160);
	sGameWindow = CGameWindow::NewL();
	sGameWindow->CreateDisplayL(gameSize.iWidth, gameSize.iHeight, 8, EOrientationNormal, 60);

	sKeyboardHandler = CKeyboardHandler::NewL();
	sGameWindow->SetKeyboardObserver(sKeyboardHandler);

	TSize screenSize = sGameWindow->GameScreen()->ScreenSize();
	PsionBpp = CGameBitmap::GetColorDepth(sGameWindow->GameScreen()->DisplayMode());
	fb.ptr = sGameWindow->GameScreen()->FrameBufferAddress();
	fb.w = screenSize.iWidth;
	fb.h = screenSize.iHeight;
	fb.pelsize = 1;
	fb.pitch = sGameWindow->GameScreen()->FrameBufferBytesPerScanLine();
	fb.indexed = 1;
	fb.cc[0].r = fb.cc[1].r = fb.cc[2].r = 8;
	fb.cc[0].l = fb.cc[1].l = fb.cc[2].l = 0;
	fb.yuv = 0;
	fb.enabled = 1;
	fb.dirty = 0;

#ifdef __WINS__
	sGameWindow->GameScreen()->SetPosition(TPoint((screenSize.iWidth - gameSize.iWidth) >> 1, (screenSize.iHeight - gameSize.iHeight) >> 1));
//	sGameWindow->GameScreen()->SetPosition(TPoint(0, 0));
#endif

	video_setup((unsigned*)fb.ptr, fb.w, fb.h, 8, 0, 0xff);
	profiler_start();
}

void vid_close()
{
}

void vid_settitle(char *title)
{
}

void vid_setpal(int i, int r, int g, int b)
{
}


static TInt sScanLine = 0;
static TInt sScanLineSkip = 1;
static TBool sFrameEnabled = EFalse;
void vid_begin()
{
	sFrameEnabled = (sFrameEnabled) ? (TBool) EFalse : (TBool)ETrue;
}




void vid_end()
{
	if (sGameWindow)
	{
#ifdef __WINS__
		sGameWindow->EmulateTargetDrawBitmap(NULL);
#endif
		sGameWindow->GameScreen()->FrameTick();
		sGameWindow->PollWServEvents();
	}
}


int osd_cycles()
{
	return User::TickCount();
}

static TBool	sShowFps = ETrue;
void vid_fps()
{
	char	fps[32];

	if (sShowFps)
	{
		sprintf(fps, "fps: %d", sGameWindow->GameScreen()->Fps());
//		video_clear_area(0, 20, 15, 1);
//		video_puts_pos(fps, 0, 20);
	}
}


byte pal_getcolor(int c, int r, int g, int b)
{
	return sGameWindow->GameScreen()->FindEpocRgbColor(r, g, b);
}

int vid_shouldrefreshline()
{
	return sFrameEnabled;
}